/** Rill v0.0.584 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var GeneratorFunction, Request, Response, Rill, co, compose, expect, http, pathToReg, respond,
  slice = [].slice;

co = require("co");

pathToReg = require("path-to-regexp");

compose = require("./util/compose");

expect = require("./util/expect");

Request = require("./request");

Response = require("./response");

respond = require("./util/respond");

http = (require("./util/browser"));

GeneratorFunction = (regeneratorRuntime.mark(function callee$0$0() {
  return regeneratorRuntime.wrap(function callee$0$0$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
    case 0:
      return context$1$0.abrupt("return");
    case 1:
    case "end":
      return context$1$0.stop();
    }
  }, callee$0$0, this);
})).constructor;

Rill = (function() {

  /*
  	 * Creates an isomorphic Rill application that can listen for browser and server page requests.
  	#
  	 * @constructor
   */
  var j, len, method, ref;

  function Rill() {
    if (!(this instanceof Rill)) {
      return new Rill();
    }
    this.middleware = [];
  }


  /*
  	 * Starts the server or browser hijacker.
   */

  Rill.prototype.listen = function() {
    var fn, ref;
    fn = co.wrap(compose([respond].concat(this.middleware)));
    return (ref = http.createServer(function(req, res) {
      if (res == null) {
        res = {};
      }
      res.statusCode = 404;
      return fn.call({}, req = new Request(req), new Response(res)).then(function() {
        return {
          redirected: Boolean(req.redirectURL)
        };
      })["catch"](function(err) {
        throw err;
      });
    })).listen.apply(ref, arguments);
  };


  /*
  	 * Simple syntactic sugar for functions that wish to modify the current application instance.
  	#
  	 * @param {Function} transformer
   */

  Rill.prototype.setup = function(transformer) {
    transformer(this);
    return this;
  };


  /*
  	 * Attaches a seperate rill application at a given path.
  	 * req.path and req.params will be updated to remove the provided path.
  	#
  	 * @param {Rill} app
   */

  Rill.prototype.mount = function(path, app) {
    var handlers;
    if (typeof path !== "string") {
      app = path;
      path = "/";
    }
    if (path === "/") {
      this.middleware = this.middleware.concat(app.middleware);
      return this;
    }
    expect(path[0] === "/", "Mount path must begin with '/'.");
    expect(path.slice(-1) !== "/", "Mount path must not end with '/'.");
    expect(app.middleware, "Can only mount Rill applications.");
    handlers = compose(app.middleware);
    this.middleware.push(regeneratorRuntime.mark(function callee$2$0(req, res, next) {
      var mounted, newPath, prevPath;

      return regeneratorRuntime.wrap(function callee$2$0$(context$3$0) {
        while (1) switch (context$3$0.prev = context$3$0.next) {
        case 0:
          if (!(req.path.indexOf(path) !== 0)) {
            context$3$0.next = 4;
            break;
          }

          context$3$0.next = 3;
          return next;
        case 3:
          return context$3$0.abrupt("return", context$3$0.sent);
        case 4:
          prevPath = req.path;
          newPath = req.path = req.path.replace(path, "");
          mounted = req.mounted = path;

          return context$3$0.delegateYield(
            handlers.call(this, req, res, (regeneratorRuntime.mark(function callee$3$0() {
              return regeneratorRuntime.wrap(function callee$3$0$(context$4$0) {
                while (1) switch (context$4$0.prev = context$4$0.next) {
                case 0:
                  req.mounted = void 0;
                  req.path = prevPath;
                  context$4$0.next = 4;
                  return next;
                case 4:
                  req.mounted = mounted;
                  return context$4$0.abrupt("return", req.path = newPath);
                case 6:
                case "end":
                  return context$4$0.stop();
                }
              }, callee$3$0, this);
            }))()),
            "t0",
            8
          );
        case 8:
          req.mounted = void 0;
          return context$3$0.abrupt("return", req.path = prevPath);
        case 10:
        case "end":
          return context$3$0.stop();
        }
      }, callee$2$0, this);
    }));
    return this;
  };


  /*
  	 * Create functions for each http verb that will accept a path url and a middleware function to use.
  	 * A "use" method is also attached that will not check the request method.
  	#
  	 * @param {String} path
  	 * @param {Generator...} handlers
   */

  ref = ["USE", "OPTIONS", "HEAD", "GET", "PUT", "POST", "DELETE"];
  for (j = 0, len = ref.length; j < len; j++) {
    method = ref[j];
    Rill.prototype[method.toLowerCase()] = (function(method) {
      return function() {
        var handler, handlers, k, keys, len1, path;
        path = arguments[0], handlers = 2 <= arguments.length ? slice.call(arguments, 1) : [];
        if (!(typeof path === "string" || path instanceof RegExp)) {
          handlers = [path].concat(handlers);
          path = "/";
        }
        for (k = 0, len1 = handlers.length; k < len1; k++) {
          handler = handlers[k];
          expect(handler instanceof GeneratorFunction, "Can only use GeneratorFunctions.");
        }
        if (path === "/" && method === "USE") {
          this.middleware = this.middleware.concat(handlers);
          return this;
        }
        expect(path[0] === "/", "At path must begin with '/'.");
        path = pathToReg(path, keys = []);
        handlers = compose(handlers);
        this.middleware.push(regeneratorRuntime.mark(function callee$3$0(req, res, next) {
          var i, l, len2, match, name, optional, ref1;

          return regeneratorRuntime.wrap(function callee$3$0$(context$4$0) {
            while (1) switch (context$4$0.prev = context$4$0.next) {
            case 0:
              if ((req.method === method || method === "USE") && (match = req.path.match(path))) {
                context$4$0.next = 4;
                break;
              }

              context$4$0.next = 3;
              return next;
            case 3:
              return context$4$0.abrupt("return", context$4$0.sent);
            case 4:
              if (req.params == null) {
                req.params = {};
              }
              i = l = 0, len2 = keys.length;
            case 6:
              if (!(l < len2)) {
                context$4$0.next = 15;
                break;
              }

              ref1 = keys[i], name = ref1.name, optional = ref1.optional;

              if (((req.params[name] = match[i + 1]) != null) || optional) {
                context$4$0.next = 12;
                break;
              }

              context$4$0.next = 11;
              return next;
            case 11:
              return context$4$0.abrupt("return", context$4$0.sent);
            case 12:
              i = ++l;
              context$4$0.next = 6;
              break;
            case 15:
              return context$4$0.delegateYield(handlers.call(this, req, res, next), "t1", 16);
            case 16:
              return context$4$0.abrupt("return", context$4$0.t1);
            case 17:
            case "end":
              return context$4$0.stop();
            }
          }, callee$3$0, this);
        }));
        return this;
      };
    })(method);
  }

  return Rill;

})();

module.exports = Rill;
