/** Rill v0.0.584 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, element, event, expect, handleForm, handleLink, handleURLChange, handler, parseURL, referrer, serialize;

event = require("component-event");

serialize = require("@rill/form-json");

parseURL = require("@rill/parse-url");

expect = require("../util/expect");

referrer = document.referrer;

handler = function() {};

element = null;

browser = {

  /*
  	 * Listen to all url change events on a dom element and trigger the server callback.
  	#
  	 * @param {DOMNode} element
  	 * @api private
   */
  listen: function(entity) {
    if (entity == null) {
      entity = global;
    }
    expect(element == null, "Can only run one server at a time.");
    element = entity;
    event.bind(element, "DOMContentLoaded", handleURLChange);
    event.bind(element, "popstate", handleURLChange);
    event.bind(element, "submit", handleForm);
    event.bind(element, "click", handleLink);
  },

  /*
  	 * Add a handler to listen to url changes.
  	#
  	 * @param {Function} fn
  	 * @api private
   */
  createServer: function(fn) {
    handler = fn;
    return browser;
  },

  /*
  	 * Trigger the registered handler to navigate to a given url.
  	#
  	 * @param {String} url
  	 * @param {Object} options
  	 * @api private
   */
  goTo: function(url, arg) {
    var body, files, headers, method, ref, replaceState, title;
    ref = arg != null ? arg : {}, method = ref.method, headers = ref.headers, body = ref.body, files = ref.files, title = ref.title, replaceState = ref.replaceState;
    if (title == null) {
      title = "";
    }
    if (headers == null) {
      headers = {};
    }
    headers.referer = referrer;
    referrer = url;
    return handler({
      url: url,
      body: body,
      files: files,
      method: method != null ? method : "GET",
      headers: headers
    }).then(function(arg1) {
      var redirected;
      redirected = (arg1 != null ? arg1 : {}).redirected;
      if (!redirected) {
        if (url.slice(-1) === "#") {
          element.scrollIntoView();
        }
        if (replaceState) {
          history.replaceState(null, title, url);
        } else {
          history.pushState(null, title, url);
        }
      }
    });
  }
};


/*
 * Handle an event that changed the url (popstate or page load)
#
 * @param {Object} event
 */

handleURLChange = function(event) {
  return browser.goTo(location.href, {
    method: "GET",
    replaceState: true
  });
};


/*
 * Handle intercepting link clicks to update the url.
#
 * @param {Object} event
 */

handleLink = function(event) {
  var el, host, protocol, ref, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey || event.button !== 0) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "A") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  if (Boolean(el.attributes.download)) {
    return;
  }
  url = el.href;
  ref = parseURL(url, el), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  browser.goTo(url);
  return event.preventDefault();
};


/*
 * Handle intercepting forms to update the url.
#
 * @param {Object} event
 */

handleForm = function(event) {
  var body, el, files, host, protocol, ref, ref1, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "FORM") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  url = el.action;
  ref = parseURL(url, location), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  ref1 = serialize(el), body = ref1.body, files = ref1.files;
  browser.goTo(url, {
    method: el.getAttribute("method") || el.method,
    body: body,
    files: files
  });
  return event.preventDefault();
};

module.exports = browser;
