/** Rill v0.0.584 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, byteLength, htmlReg, isType, statuses;

isType = require("@rill/is-type");

{
  browser = require("./browser");
}

htmlReg = /^\s*</;

statuses = {
  empty: {
    204: true,
    205: true,
    304: true
  },
  redirect: {
    300: true,
    301: true,
    302: true,
    303: true,
    305: true,
    307: true,
    308: true
  }
};


/*
 * Calculate bytelengths for complex strings.
#
 * @param {String} str
 * @return {Number}
 */

byteLength = function(str) {
  var char, code, i, len, len1;
  if (!str) {
    return 0;
  }
  if (typeof str.pipe === "function") {
    return;
  }
  str = str.toString();
  len = str.length;
  for (i = 0, len1 = str.length; i < len1; i++) {
    char = str[i];
    code = char.charCodeAt();
    if ((0x7f < code && code <= 0x7ff)) {
      len += 1;
    } else if ((0x7ff < code && code <= 0xffff)) {
      len += 2;
    } else if ((0xDC00 <= code && code <= 0xDFFF)) {
      len -= 1;
    }
  }
  return len;
};


/*
 * A default middleware that handles sending the response for the request.
#
 * @param {Object} req
 * @param {Object} res
 * @param {Function} next
 */

module.exports = regeneratorRuntime.mark(function callee$0$0(req, res, next) {
  var body, cookie, cookies, headers, i, len1, method, originalRes, redirectURL, status;

  return regeneratorRuntime.wrap(function callee$0$0$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
    case 0:
      return context$1$0.delegateYield(next, "t3", 1);
    case 1:
      if (!res.isSent()) {
        context$1$0.next = 3;
        break;
      }

      return context$1$0.abrupt("return");
    case 3:
      method = req.method, redirectURL = req.redirectURL;
      originalRes = res.res, headers = res.headers, status = res.status, body = res.body;
      cookies = req.cookies.serialize(true);
      if (cookies) {
        for (i = 0, len1 = cookies.length; i < len1; i++) {
          cookie = cookies[i];
          document.cookie = cookie;
        }
      }

      if (!(redirectURL != null)) {
        context$1$0.next = 11;
        break;
      }

      if (headers["location"] == null) {
        headers["location"] = redirectURL;
      }
      if (!statuses.redirect[status]) {
        status = 302;
      }
      return context$1$0.abrupt("return", browser.goTo(redirectURL));
    case 11:
      if (Number(status) === 404 && (body != null)) {
        status = 200;
      }
      if (statuses.empty[status] || (body == null)) {
        if (headers["content-type"] == null) {
          headers["content-type"] = void 0;
        }
        body = void 0;
      } else if (isType.String(body)) {
        if (headers["content-type"] == null) {
          headers["content-type"] = "text/" + (htmlReg.test(body) ? "html" : "plain") + "; charset=UTF-8";
        }
      } else if (isType.Buffer(body)) {
        headers["content-length"] = body.length;
      } else if (isType.Stream(body)) {
        if (headers["content-type"] == null) {
          headers["content-type"] = "application/octet-stream";
        }
      } else {
        if (headers["content-type"] == null) {
          headers["content-type"] = "application/json; charset=UTF-8";
        }
        try {
          body = JSON.stringify(body);
        } catch (_error) {}
      }
    case 13:
    case "end":
      return context$1$0.stop();
    }
  }, callee$0$0, this);
});
