/** Rill v0.0.584 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, byteLength, htmlReg, isType, statuses;

isType = require("@rill/is-type");

htmlReg = /^\s*</;

statuses = {
  empty: {
    204: true,
    205: true,
    304: true
  },
  redirect: {
    300: true,
    301: true,
    302: true,
    303: true,
    305: true,
    307: true,
    308: true
  }
};


/*
 * Calculate bytelengths for complex strings.
#
 * @param {String} str
 * @return {Number}
 */

byteLength = function(str) {
  var char, code, i, len, len1;
  if (!str) {
    return 0;
  }
  if (typeof str.pipe === "function") {
    return;
  }
  str = str.toString();
  len = str.length;
  for (i = 0, len1 = str.length; i < len1; i++) {
    char = str[i];
    code = char.charCodeAt();
    if ((0x7f < code && code <= 0x7ff)) {
      len += 1;
    } else if ((0x7ff < code && code <= 0xffff)) {
      len += 2;
    } else if ((0xDC00 <= code && code <= 0xDFFF)) {
      len -= 1;
    }
  }
  return len;
};


/*
 * A default middleware that handles sending the response for the request.
#
 * @param {Object} req
 * @param {Object} res
 * @param {Function} next
 */

module.exports = function*(req, res, next) {
  var body, cookie, cookies, headers, i, len1, method, originalRes, redirectURL, status;
  (yield* next);
  if (res.isSent()) {
    return;
  }
  method = req.method, redirectURL = req.redirectURL;
  originalRes = res.res, headers = res.headers, status = res.status, body = res.body;
  {
    cookies = req.cookies.serialize();
    if (cookies) {
      headers["set-cookie"] = cookies;
    }
  }
  if (redirectURL != null) {
    if (headers["location"] == null) {
      headers["location"] = redirectURL;
    }
    if (!statuses.redirect[status]) {
      status = 302;
    }
  }
  if (Number(status) === 404 && (body != null)) {
    status = 200;
  }
  if (statuses.empty[status] || (body == null)) {
    if (headers["content-type"] == null) {
      headers["content-type"] = void 0;
    }
    body = void 0;
  } else if (isType.String(body)) {
    if (headers["content-type"] == null) {
      headers["content-type"] = "text/" + (htmlReg.test(body) ? "html" : "plain") + "; charset=UTF-8";
    }
  } else if (isType.Buffer(body)) {
    headers["content-length"] = body.length;
  } else if (isType.Stream(body)) {
    if (headers["content-type"] == null) {
      headers["content-type"] = "application/octet-stream";
    }
  } else {
    if (headers["content-type"] == null) {
      headers["content-type"] = "application/json; charset=UTF-8";
    }
    try {
      body = JSON.stringify(body);
    } catch (_error) {}
  }
  {
    if ("HEAD" === method) {
      originalRes.writeHead(status, headers);
      return originalRes.end();
    } else if (isType.Stream(body)) {
      originalRes.writeHead(status, headers);
      return body.pipe(originalRes);
    } else {
      if (headers["content-length"] == null) {
        headers["content-length"] = byteLength(body);
      }
      originalRes.writeHead(status, headers);
      return originalRes.end(body);
    }
  }
};
