Cookies  = require("@rill/cookies")
parseURL = require("@rill/parse-url")
expect   = require("./util/expect")

###
# Constructor that will normalize a request object.
#
# @param {Object} req
# @constructor
###
class Request
	constructor: (req)->
		return new Request(req) unless this instanceof Request
		expect(req?, "Could not create Request with an empty object.")
		expect(req.url?, "Could not create a Request without a URL.")
		@req     = req
		@body    = req.body ? {}
		@files   = req.files ? []
		@headers = req.headers ? {}
		@method  = (req.method ? "GET").toUpperCase()

		if process.browser
			@ip             = "127.0.0.1"
			@headers.cookie = document.cookie
			defaults        = location
		else
			@ip = @headers["x-forwarded-for"]?.split(",")[0] ? req.connection?.remoteAddress
			[@hostname, @port] = (@headers["x-forwarded-host"] or @headers["host"] or "").split(":")
			@protocol = if req.socket?.encrypted then "https:" else "http:"
			defaults = this

		@cookies = new Cookies(@headers.cookie)
		@[key] = val for key, val of parseURL(req.url, defaults)

	###
	# Trigger another request that will redirect the browser and send a location change on the server.
	# If "back" is the url then it will attempt to redirect the user back a url, otherwise the alt will be used.
	#
	# @param {String} url
	# @param {String} alt
	###
	redirect: (url="back", alt="/")->
		if @mounted
			alt = @mounted + alt if alt[0] is "/"
			url = @mounted + url if url[0] is "/"

		@redirectURL = (
			unless url is "back" then url
			else @headers.referer or alt
		)

module.exports = Request
