isType   = require("@rill/is-type")
browser  = require("./browser") if process.browser
htmlReg  = /^\s*</
statuses =
	empty:
		204: true
		205: true
		304: true
	redirect:
		300: true
		301: true
		302: true
		303: true
		305: true
		307: true
		308: true

###
# Calculate bytelengths for complex strings.
#
# @param {String} str
# @return {Number}
###
byteLength = (str)->
	return 0 unless str
	return if typeof str.pipe is "function"
	str = str.toString()
	len = str.length
	for char in str
		code = char.charCodeAt()
		if 0x7f < code <= 0x7ff
			len += 1
		else if 0x7ff < code <= 0xffff
			len += 2
		else if 0xDC00 <= code <= 0xDFFF
			len -= 1
	return len


###
# A default middleware that handles sending the response for the request.
#
# @param {Object} req
# @param {Object} res
# @param {Function} next
###
module.exports = (req, res, next)->
	yield from next
	# Skip request if something had already ended it.
	return if res.isSent()

	{ method, redirectURL } = req
	{ res: originalRes, headers, status, body } = res

	# Handle cookies isomorphically.
	if process.browser
		cookies = req.cookies.serialize(true)
		document.cookie = cookie for cookie in cookies if cookies
	else
		cookies = req.cookies.serialize()
		headers["set-cookie"] = cookies if cookies

	# Redirects isomorphically.
	if redirectURL?
		headers["location"] ?= redirectURL
		status = 302 unless statuses.redirect[status]
		return browser.goTo(redirectURL) if process.browser

	# Default the status to 200 if there is substance to the response.
	status = 200 if Number(status) is 404 and body?

	if statuses.empty[status] or not body?
		headers["content-type"] ?= undefined
		body = undefined

	else if isType.String(body)
		headers["content-type"] ?= "text/#{if htmlReg.test(body) then "html" else "plain"}; charset=UTF-8"

	else if isType.Buffer(body)
		headers["content-length"] = body.length

	else if isType.Stream(body)
		headers["content-type"] ?= "application/octet-stream"

	else
		headers["content-type"] ?= "application/json; charset=UTF-8"
		try body = JSON.stringify(body)

	unless process.browser
		if "HEAD" is method
			originalRes.writeHead(status, headers)
			originalRes.end()
		else if isType.Stream(body)
			originalRes.writeHead(status, headers)
			body.pipe(originalRes)
		else
			headers["content-length"] ?= byteLength(body)
			originalRes.writeHead(status, headers)
			originalRes.end(body)

